@@ -2,7 +2,6 @@
 
 from __future__ import annotations
 
-import enum
 import os
 from collections import defaultdict
 from typing import Any, TYPE_CHECKING, Optional
@@ -40,14 +39,6 @@ class UUIDGenerator:
             return shortuuid.uuid()[:8]
 
 
-class EventType(enum.Enum):
-    JumpBackToLoopStart = 0
-    InitialValue = 1
-    Binding = 2
-    Mutation = 3
-    Deletion = 4
-
-
 @attr.s(auto_attribs=True)
 class Event:
     lineno: int
@@ -202,7 +193,7 @@ class JumpBackToLoopStart(Event):
 
     Not like other types of events, JumpBackToLoopStart is irrelevant to any identifier,
     nor is it showed on the trace graph. We need it to help track iterations of loops.
-    See loop.js for more information.
+    See trace_graph.js for more information.
     """
 
     # The jump target's offset, which is guaranteed to be a loop start.
