@@ -81,19 +81,21 @@ class Tracer:
             if raw_frame.f_code.co_name == "<module>"
             else raw_frame.f_code.co_name
         )
+        instructions = {
+            instr.offset: instr for instr in dis.get_instructions(raw_frame.f_code)
+        }
         self.frame = Frame(
             # For testing, only stores the basename so it's separator agnostic.
             filename=utils.shorten_path(
                 raw_frame.f_code.co_filename, 1 if utils.run_in_test() else 3
             ),
             frame_name=frame_name,
+            instructions=instructions,
             offset_to_lineno=utils.map_bytecode_offset_to_lineno(raw_frame),
         )
         FrameTree.add_frame(self.frame.frame_id, self.frame)
         self.frame_logger = logger.FrameLogger(
-            instructions={
-                instr.offset: instr for instr in dis.get_instructions(raw_frame.f_code)
-            },
+            instructions=instructions,
             initial_instr_pointer=initial_instr_pointer,
             frame=self.frame,
             debug_mode=self.debug_mode,
@@ -214,35 +216,28 @@ class Tracer:
         """Test only. Provides access to logged events."""
         return list(self.frame_logger.frame.loops.values())
 
-    @property
-    def global_tracer(self):
-        def _global_tracer(raw_frame, event, arg):
-            # Later when we need to trace more functions, we should identify those
-            # functions or at least use utils.should_exclude(frame) to avoid tracing
-            # unnecessary frames.
-            #
-            # self.tracer_state == TracerFSM.INITIAL is for preventing stepping into
-            # recursive calls, since their f_code are the same.
-            if (
-                event == "call"
-                and id(raw_frame.f_code) == self.decorated_function_code_id
-                and self.tracer_state == TracerFSM.INITIAL
-            ):
-                raw_frame.f_trace_opcodes = True
-                self._initialize_frame_and_logger(raw_frame, initial_instr_pointer=0)
-                return self.local_tracer
+    def global_tracer(self, raw_frame, event, arg):
+        # Later when we need to trace more functions, we should identify those
+        # functions or at least use utils.should_exclude(frame) to avoid tracing
+        # unnecessary frames.
+        #
+        # self.tracer_state == TracerFSM.INITIAL is for preventing stepping into
+        # recursive calls, since their f_code are the same.
+        if (
+            event == "call"
+            and id(raw_frame.f_code) == self.decorated_function_code_id
+            and self.tracer_state == TracerFSM.INITIAL
+        ):
+            raw_frame.f_trace_opcodes = True
+            self._initialize_frame_and_logger(raw_frame, initial_instr_pointer=0)
+            return self.local_tracer
 
-        return _global_tracer
-
-    @property
-    def local_tracer(self):
-        def _local_tracer(raw_frame, event, arg):
-            if utils.should_exclude(raw_frame):
-                return
-            if event == "opcode":
-                self.frame_logger.update(raw_frame)
-            if event == "return":
-                # print(raw_frame, event, arg, raw_frame.f_lasti)
-                self.frame.log_return_event(raw_frame, value=arg)
-
-        return _local_tracer
+    def local_tracer(self, raw_frame, event, arg):
+        if utils.should_exclude(raw_frame):
+            return
+        if event == "opcode":
+            # print(raw_frame, event, arg, raw_frame.f_lasti)
+            self.frame_logger.update(raw_frame)
+        if event == "return":
+            # print(raw_frame, event, arg, raw_frame.f_lasti)
+            self.frame.log_return_event(raw_frame, value=arg)
