@@ -16,6 +16,8 @@ from types import FrameType
 from typing import Any, Optional
 
 import jsonpickle
+import jsonpickle.ext.numpy as jsonpickle_numpy
+import jsonpickle.ext.pandas as jsonpickle_pandas
 import more_itertools
 from cheap_repr import cheap_repr
 from pygments import highlight
@@ -25,6 +27,11 @@ from pygments.lexers import PythonLexer
 _INSTALLATION_PATHS = list(sysconfig.get_paths().values())
 _PYTHON_EXECUTABLE_PATH = sys.executable
 
+# Make sure Numpy and Pandas objects can be correctly encoded.
+# https://github.com/jsonpickle/jsonpickle#numpy-support
+jsonpickle_numpy.register_handlers()
+jsonpickle_pandas.register_handlers()
+
 
 def should_ignore_event(
     *, target: str, value: Any, frame: Optional[FrameType] = None
@@ -213,6 +220,13 @@ def shorten_path(file_path, length):
     return str(Path(*Path(file_path).parts[-length:]))
 
 
+def all_none(*args):
+    for arg in args:
+        if arg is not None:
+            return False
+    return True
+
+
 if __name__ == "__main__":
     # For development.
     print(computed_gotos_enabled())
